package com.jcg;

import com.google.gson.Gson;
import com.google.gson.internal.LinkedTreeMap;
import com.google.gson.reflect.TypeToken;
import org.apache.http.HttpHost;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.index.query.QueryBuilders;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;

public class ElasticSearchHighLevelRestClient {
    private static final Path flightDataJsonFile =
            Paths.get("src/main/resources/flightdata/2016-07-01-1300Z.json");
    private static final RestHighLevelClient restClient = new RestHighLevelClient(
            RestClient.builder(new HttpHost("localhost", 9200, "http")));

    public static void main(String[] args) {
        BulkRequest request = addDocumentsToIndex("flights");
        try {
            restClient.bulk(request, RequestOptions.DEFAULT);
            System.out.println("/flight/_count:" + countDocuments("flights"));
            restClient.close();
        } catch (IOException e) {
            System.err.println(e.getStackTrace());
        }
    }

    @SuppressWarnings("unchecked")
    private static BulkRequest addDocumentsToIndex(final String index) {
        final Gson gson = new Gson();
        BulkRequest request = new BulkRequest(index);
        try (BufferedReader reader = Files.newBufferedReader(flightDataJsonFile)) {
            Map<String, Object> map = gson.fromJson(reader,
                    new TypeToken<Map<String, Object>>() {
                    }.getType());
            List<Object> acList = (List<Object>) (map.get("acList"));
            for (Object item : acList) {
                LinkedTreeMap<String, Object> flight = (LinkedTreeMap<String, Object>) item;
                final long id = Double.valueOf(flight.get("Id").toString()).longValue();
                flight.remove("Id");
                request.add(new IndexRequest("flights")
                        .id(String.valueOf(id))
                        .source(flight));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return request;
    }

    private static long countDocuments(String index) {
        long numOfDocuments = 0;
        CountRequest countRequest = new CountRequest(index);
        countRequest.query(QueryBuilders.matchAllQuery());
        try {
            CountResponse countResponse = restClient
                    .count(countRequest, RequestOptions.DEFAULT);
            numOfDocuments = countResponse.getCount();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return numOfDocuments;
    }

}
