package com.jcg;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;
import org.apache.http.Header;
import org.apache.http.HttpHost;
import org.apache.http.message.BasicHeader;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

public class ElasticSearchLowLevelRestClient {
    private static final Path flightDataJsonFile =
            Paths.get("src/main/resources/flightdata/2016-07-01-1300Z.json");
    private static final Gson gson = new Gson();

    public static void main(String[] args) {
//        String jsonDoc = "{\"Rcvr\":1,\"HasSig\":false,\"Icao\":\"494102\",\"Bad\":false,\"Reg\":\"CS-PHB\",\"FSeen\":\"/Date(1467378028852)/\",\"TSecs\":1,\"CMsgs\":1,\"Alt\":10750,\"GAlt\":10535,\"InHg\":29.7047253,\"AltT\":0,\"Call\":\"NJE785L\",\"Lat\":52.601028,\"Long\":-8.849945,\"PosTime\":1467378028852,\"Mlat\":false,\"Tisb\":false,\"Spd\":294.0,\"Trak\":103.0,\"TrkH\":false,\"Type\":\"E55P\",\"Mdl\":\"Embraer Phenom 300\",\"Man\":\"Embraer\",\"CNum\":\"50500209\",\"Op\":\"NetJets Europe\",\"OpIcao\":\"NJE\",\"Sqk\":\"\",\"Vsi\":3840,\"VsiT\":0,\"WTC\":2,\"Species\":1,\"Engines\":\"2\",\"EngType\":3,\"EngMount\":1,\"Mil\":false,\"Cou\":\"Portugal\",\"HasPic\":false,\"Interested\":false,\"FlightsCount\":0,\"Gnd\":false,\"SpdTyp\":0,\"CallSus\":false,\"ResetTrail\":true,\"TT\":\"a\",\"Trt\":2,\"Year\":\"2014\",\"Cos\":[52.601028,-8.849945,1467378028852.0,null]}";
        RestClient restClient = getRestClient();
        try (BufferedReader reader = Files.newBufferedReader(flightDataJsonFile)) {
            Map<String, JsonElement> map = gson.fromJson(reader,
                    new TypeToken<Map<String, JsonElement>>() {
                    }.getType());
            JsonElement acList = (map.get("acList"));
            JsonArray array = acList.getAsJsonArray();
            indexJsonDocuments(restClient, array);
//            indexJsonDocumentsParallel(restClient, array);
            System.out.println("/flight/_count" + countDocuments(restClient, "flights"));
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                restClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private static void indexJsonDocuments(RestClient restClient, JsonArray array) throws IOException {
        for (JsonElement item : array) {
            JsonObject itemAsJsonObject = item.getAsJsonObject();
            String id = itemAsJsonObject.remove("Id").getAsString();
            Request request = new Request("POST", "/flights/_doc/" + id);
            String jsonDoc = item.toString();
            jsonDoc = jsonDoc.replaceAll("\"", "\\\"");
            request.setJsonEntity(jsonDoc);
            Response response = restClient.performRequest(request);
            if (response.getStatusLine().getStatusCode() != 200) {
                System.err.println("Could not add document with Id: " + id + " to index /flight");
            }
        }
    }

    private static void indexJsonDocumentsParallel(RestClient restClient, JsonArray array) throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(array.size());
        for (JsonElement item : array) {
            JsonObject itemAsJsonObject = item.getAsJsonObject();
            String id = itemAsJsonObject.remove("Id").getAsString();
            Request request = new Request("POST", "/flights/_doc/" + id);
            String jsonDoc = item.toString();
            jsonDoc = jsonDoc.replaceAll("\"", "\\\"");
            request.setJsonEntity(jsonDoc);
            Cancellable cancellable = restClient.performRequestAsync(
                    request,
                    new ResponseListener() {
                        @Override
                        public void onSuccess(Response response) {
                            System.out.println("Document with Id: " + id + " was successfully added to index /flight");
                            latch.countDown();
                        }

                        @Override
                        public void onFailure(Exception exception) {
                            System.err.println("Could not add document with Id: " + id + " to index /flight");
                            latch.countDown();
                        }
                    }
            );
        }
        latch.await();
    }

    private static RestClient getRestClient() {
        return RestClient.builder(
                new HttpHost("localhost", 9200, "http"))
                .setDefaultHeaders(new Header[]{
                        new BasicHeader("accept", "application/json"),
                        new BasicHeader("content-type", "application/json")})
                .setFailureListener(new RestClient.FailureListener() {
                    public void onFailure(Node node) {
                        System.err.println("Low level Rest Client Failure on node " +
                                node.getName());
                    }
                }).build();
    }

    private static String countDocuments(RestClient restClient, String index) throws IOException {
        Request request = new Request("GET", "/" + index + "/_count");
        Response response = restClient.performRequest(request);
        String responseBody = EntityUtils.toString(response.getEntity());
        final String[] entries = responseBody.split(","); // {"count":5900,...
        return entries[0].substring(8);
    }
}
