package com.jcg;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.internal.LinkedTreeMap;
import com.google.gson.reflect.TypeToken;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class JsonFlightFileReader {
    private static final String flightDataJsonFile = "src/main/resources/flightdata/2016-07-01-1300Z.json";
    private static final Gson gson = new Gson();

    public static void main(String[] args) {
//        parseJsonFile(flightDataJsonFile);
        parseJsonFile2(flightDataJsonFile);
    }

    private static void parseJsonFile(String file) {
        try (BufferedReader reader = Files.newBufferedReader(Paths.get(file))) {
            Map<String, Object> map = gson.fromJson(reader, new TypeToken<Map<String, Object>>() {
            }.getType());
            List<Object> acList = (List<Object>) (map.get("acList"));
            for (Object item : acList) {
                LinkedTreeMap<String, Object> flight = (LinkedTreeMap<String, Object>) item;
                for (Map.Entry<String, Object> entry : flight.entrySet()) {
                    String key = entry.getKey();
                    Object value = entry.getValue();
                    String outEntry = (key.equals("Id") ? "{" + key : key) + " : " + value + ", ";
                    System.out.print(outEntry);
                }
                System.out.println("}");
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void parseJsonFile2(String file) {
        try (BufferedReader reader = Files.newBufferedReader(Paths.get(flightDataJsonFile))) {
            Map<String, JsonElement> map = gson.fromJson(reader,
                    new TypeToken<Map<String, JsonElement>>() {
                    }.getType());

            JsonElement acList = (map.get("acList"));
            JsonArray array = acList.getAsJsonArray();

            for (JsonElement item : array) {
                JsonObject itemAsJsonObject = item.getAsJsonObject();
                String id = itemAsJsonObject.get("Id").getAsString();

                StringBuilder recordAsString = new StringBuilder();
                Set<Map.Entry<String, JsonElement>> entries = item.getAsJsonObject().entrySet();
                for (Map.Entry<String, JsonElement> entry : entries) {
                    if (entry.getKey().equals("Id")) {
                        recordAsString.append("{");
                    } else {
                        recordAsString.append(entry.getKey()).append(" : ").append(entry.getValue()).append(",");
                    }
                }
                recordAsString.setCharAt(recordAsString.length() - 1, '}');
                System.out.println(recordAsString.toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
